@@ -7,12 +7,24 @@ import dis
 import sys
 import inspect
 from collections import namedtuple
-from numba import utils
+from numba import utils, targets
 from numba.config import PYVERSION
 
 opcode_info = namedtuple('opcode_info', ['argsize'])
 
 
+def get_function_object(obj):
+    """
+    Objects that wraps function should provide a "__numba__" magic attribute
+    that contains a name of an attribute that contains the actual python
+    function object.
+    """
+    attr = getattr(obj, "__numba__", None)
+    if attr:
+        return getattr(obj, attr)
+    return obj
+
+
 def get_code_object(obj):
     "Shamelessly borrowed from llpython"
     return getattr(obj, '__code__', getattr(obj, 'func_code', None))
@@ -44,6 +56,10 @@ def _make_bytecode_table():
             ('SLICE+1', 0),
             ('SLICE+2', 0),
             ('SLICE+3', 0),
+            ('STORE_SLICE+0', 0),
+            ('STORE_SLICE+1', 0),
+            ('STORE_SLICE+2', 0),
+            ('STORE_SLICE+3', 0),
         ]
     elif sys.version_info[0] == 3:
         version_specific += [
@@ -121,6 +137,7 @@ def _as_opcodes(seq):
             lst.append(c)
     return lst
 
+
 BYTECODE_TABLE = _make_bytecode_table()
 
 JREL_OPS = frozenset(dis.hasjrel)
@@ -265,6 +282,7 @@ class CustomByteCode(ByteCodeBase):
 
 class ByteCode(ByteCodeBase):
     def __init__(self, func):
+        func = get_function_object(func)
         code = get_code_object(func)
         if not code:
             raise ByteCodeSupportError("%s does not provide its bytecode" %
