@@ -1,10 +1,8 @@
 from __future__ import print_function, division, absolute_import
-import inspect
 from collections import defaultdict
 from llvm.core import Type, Builder, Module
 import llvm.core as lc
-from numba import ctypes_support as ctypes
-from numba import ir, types, typing, cgutils, utils, config
+from numba import ir, types, cgutils, utils, config
 
 
 try:
@@ -20,9 +18,19 @@ class LoweringError(Exception):
         super(LoweringError, self).__init__("%s\n%s" % (msg, loc.strformat()))
 
 
+def default_mangler(name, argtypes):
+    codedargs = '.'.join(str(a).replace(' ', '_') for a in argtypes)
+    return '.'.join([name, codedargs])
+
+
 class FunctionDescriptor(object):
+    __slots__ = ('native', 'pymod', 'name', 'doc', 'blocks', 'typemap',
+                 'calltypes', 'args', 'kws', 'restype', 'argtypes',
+                 'qualified_name', 'mangled_name')
+
     def __init__(self, native, pymod, name, doc, blocks, typemap,
-                 restype, calltypes, args, kws):
+                 restype, calltypes, args, kws, mangler=None, argtypes=None,
+                 qualname=None):
         self.native = native
         self.pymod = pymod
         self.name = name
@@ -34,29 +42,38 @@ class FunctionDescriptor(object):
         self.kws = kws
         self.restype = restype
         # Argument types
-        self.argtypes = [self.typemap[a] for a in args]
-
-        self.qualified_name = '.'.join([self.pymod.__name__, self.name])
-        codedargs = '.'.join(str(a).replace(' ', '_') for a in self.argtypes)
-        self.mangled_name = '.'.join([self.qualified_name, codedargs])
+        self.argtypes = argtypes or [self.typemap[a] for a in args]
+        self.qualified_name = qualname or '.'.join([self.pymod.__name__,
+                                                    self.name])
+        mangler = default_mangler if mangler is None else mangler
+        self.mangled_name = mangler(self.qualified_name, self.argtypes)
 
 
 def _describe(interp):
     func = interp.bytecode.func
-    fname = func.__name__
-    pymod = inspect.getmodule(func)
+    fname = interp.bytecode.func_name
+    pymod = interp.bytecode.module
     doc = func.__doc__ or ''
     args = interp.argspec.args
     kws = ()        # TODO
     return fname, pymod, doc, args, kws
 
 
-def describe_function(interp, typemap, restype, calltypes):
+def describe_external(name, restype, argtypes):
+    args = ["arg%d" % i for i in range(len(argtypes))]
+    fd = FunctionDescriptor(native=True, pymod=None, name=name, doc='',
+                            blocks=None, restype=restype, calltypes=None,
+                            argtypes=argtypes, args=args, kws=None,
+                            typemap=None, qualname=name, mangler=lambda a,x: a)
+    return fd
+
+
+def describe_function(interp, typemap, restype, calltypes, mangler):
     fname, pymod, doc, args, kws = _describe(interp)
     native = True
     sortedblocks = utils.SortedMap(utils.dict_iteritems(interp.blocks))
     fd = FunctionDescriptor(native, pymod, fname, doc, sortedblocks,
-                            typemap, restype, calltypes, args, kws)
+                            typemap, restype, calltypes, args, kws, mangler)
     return fd
 
 
@@ -91,10 +108,12 @@ class BaseLower(object):
         self.function = context.declare_function(self.module, fndesc)
         self.entry_block = self.function.append_basic_block('entry')
         self.builder = Builder.new(self.entry_block)
+        # self.builder = cgutils.VerboseProxy(self.builder)
 
         # Internal states
         self.blkmap = {}
         self.varmap = {}
+        self.firstblk = min(self.fndesc.blocks.keys())
 
         # Subclass initialization
         self.init()
@@ -128,10 +147,12 @@ class BaseLower(object):
         self.post_lower()
         # Close entry block
         self.builder.position_at_end(self.entry_block)
-        self.builder.branch(self.blkmap[0])
+        self.builder.branch(self.blkmap[self.firstblk])
 
-        if config.DEBUG:
+        if config.DUMP_LLVM:
+            print(("LLVM DUMP %s" % self.fndesc.qualified_name).center(80,'-'))
             print(self.module)
+            print('=' * 80)
         self.module.verify()
 
     def init_argument(self, arg):
@@ -164,8 +185,11 @@ class Lower(BaseLower):
             cond = self.loadvar(inst.cond.name)
             tr = self.blkmap[inst.truebr]
             fl = self.blkmap[inst.falsebr]
-            assert cond.type == Type.int(1), ("cond is not i1: %s" % inst)
-            self.builder.cbranch(cond, tr, fl)
+
+            condty = self.typeof(inst.cond.name)
+            pred = self.context.cast(self.builder, cond, condty, types.boolean)
+            assert pred.type == Type.int(1), ("cond is not i1: %s" % pred.type)
+            self.builder.cbranch(pred, tr, fl)
 
         elif isinstance(inst, ir.Jump):
             target = self.blkmap[inst.target]
@@ -250,6 +274,10 @@ class Lower(BaseLower):
                 return self.context.make_constant_array(self.builder, ty,
                                                         value.value)
 
+            elif isinstance(ty, types.UniTuple):
+                consts = [self.context.get_constant(t, v)
+                          for t, v in zip(ty, value.value)]
+                return cgutils.pack_array(self.builder, consts)
             else:
                 raise NotImplementedError('global', ty)
 
@@ -287,14 +315,21 @@ class Lower(BaseLower):
                                      resty)
 
         elif expr.op == 'call':
-            assert not expr.kws
+
             argvals = [self.loadvar(a.name) for a in expr.args]
             argtyps = [self.typeof(a.name) for a in expr.args]
             signature = self.fndesc.calltypes[expr]
-            fnty = self.typeof(expr.func.name)
-            castvals = [self.context.cast(self.builder, av, at, ft)
-                        for av, at, ft in zip(argvals, argtyps,
-                                              signature.args)]
+
+            if isinstance(expr.func, ir.Intrinsic):
+                fnty = expr.func.name
+                castvals = expr.func.args
+            else:
+                assert not expr.kws, expr.kws
+                fnty = self.typeof(expr.func.name)
+
+                castvals = [self.context.cast(self.builder, av, at, ft)
+                            for av, at, ft in zip(argvals, argtyps,
+                                                  signature.args)]
 
             if isinstance(fnty, types.Method):
                 # Method of objects are handled differently
@@ -312,6 +347,9 @@ class Lower(BaseLower):
                 # Normal function resolution
                 impl = self.context.get_function(fnty, signature)
                 res = impl(self.builder, castvals)
+                libs = getattr(impl, "libs", ())
+                if libs:
+                    self.context.add_libs(libs)
             return self.context.cast(self.builder, res, signature.return_type,
                                      resty)
 
@@ -378,8 +416,8 @@ class Lower(BaseLower):
 
     def storevar(self, value, name):
         ptr = self.getvar(name)
-        assert value.type == ptr.type.pointee, (str(value.type),
-                                                str(ptr.type.pointee))
+        assert value.type == ptr.type.pointee,\
+            "store %s to ptr of %s" % (value.type, ptr.type.pointee)
         self.builder.store(value, ptr)
 
     def alloca(self, name, type):
@@ -572,7 +610,7 @@ class PyLower(BaseLower):
         elif expr.op == 'itervalid':
             iterstate = self.loadvar(expr.value.name)
             _, valid = self.unpack_iter(iterstate)
-            return valid
+            return self.builder.trunc(valid, Type.int(1))
         elif expr.op == 'getitem':
             target = self.loadvar(expr.target.name)
             index = self.loadvar(expr.index.name)
@@ -699,16 +737,18 @@ class PyLower(BaseLower):
     def pack_iter(self, obj):
         iterstate = PyIterState(self.context, self.builder)
         iterstate.iterator = obj
-        iterstate.valid = cgutils.true_bit
-        return iterstate._getvalue()
+        iterstate.valid = cgutils.true_byte
+        return iterstate._getpointer()
 
     def unpack_iter(self, state):
-        iterstate = PyIterState(self.context, self.builder, value=state)
+        iterstate = PyIterState(self.context, self.builder, ref=state)
         return tuple(iterstate)
 
     def set_iter_valid(self, state, item):
-        iterstate = PyIterState(self.context, self.builder, value=state)
-        iterstate.valid = cgutils.is_not_null(self.builder, item)
+        iterstate = PyIterState(self.context, self.builder, ref=state)
+        iterstate.valid = cgutils.as_bool_byte(self.builder,
+                                               cgutils.is_not_null(self.builder,
+                                                                   item))
 
         with cgutils.if_unlikely(self.builder, self.is_null(item)):
             self.check_occurred()
