@@ -5,7 +5,6 @@ the targets to choose their representation.
 from __future__ import print_function, division, absolute_import
 from collections import defaultdict
 import numpy
-import ctypes
 
 
 def _autoincr():
@@ -18,6 +17,11 @@ _typecache = defaultdict(_autoincr)
 
 
 class Type(object):
+    """
+    The default behavior is to provide equality through `name` attribute.
+    Two types are equal if there `name` are equal.
+    Subclass can refine this behavior.
+    """
     __slots__ = '_code', 'name', 'is_parametric'
 
     def __init__(self, name, param=False):
@@ -32,7 +36,7 @@ class Type(object):
         return hash(self.name)
 
     def __eq__(self, other):
-        return self is other
+        return self.name == other.name
 
     def __ne__(self, other):
         return not (self == other)
@@ -74,6 +78,15 @@ class Type(object):
 
 
 class Integer(Type):
+    def __init__(self, *args, **kws):
+        super(Integer, self).__init__(*args, **kws)
+        # Determine bitwidth
+        for prefix in ('int', 'uint'):
+            if self.name.startswith(prefix):
+                bitwidth = int(self.name[len(prefix):])
+        self.bitwidth = bitwidth
+        self.signed = self.name.startswith('int')
+
     def cast_python_value(self, value):
         return getattr(numpy, self.name)(value)
 
@@ -129,6 +142,21 @@ class Module(Type):
         return hash(self.pymod)
 
 
+class Macro(Type):
+    def __init__(self, template):
+        self.template = template
+        cls = type(self)
+        super(Macro, self).__init__("%s(%s)" % (cls.__name__, template))
+
+    def __eq__(self, other):
+        if isinstance(other, Macro):
+            return self.template == other.template
+
+    def __hash__(self):
+        # FIXME maybe this should not be hashable
+        return hash(self.template)
+
+
 class Function(Type):
     def __init__(self, template):
         self.template = template
@@ -417,14 +445,16 @@ len_type = Dummy('len')
 range_type = Dummy('range')
 slice_type = Dummy('slice')
 abs_type = Dummy('abs')
+neg_type = Dummy('neg')
 print_type = Dummy('print')
+sign_type = Dummy('sign')
 
 range_state32_type = Type('range_state32')
 range_state64_type = Type('range_state64')
 range_iter32_type = Type('range_iter32')
 range_iter64_type = Type('range_iter64')
 
-slice2_type = Type('slice2_type')
+# slice2_type = Type('slice2_type')
 slice3_type = Type('slice3_type')
 
 signed_domain = frozenset([int8, int16, int32, int64])
@@ -434,7 +464,7 @@ real_domain = frozenset([float32, float64])
 complex_domain = frozenset([complex64, complex128])
 number_domain = real_domain | integer_domain | complex_domain
 
-# Aliases
+# Aliases to Numpy type names
 
 b1 = bool_
 i1 = int8
@@ -456,16 +486,20 @@ float_ = float32
 double = float64
 void = none
 
-char = globals()["int%d" % (ctypes.sizeof(ctypes.c_char) * 8)]
-uchar = byte = globals()["uint%d" % (ctypes.sizeof(ctypes.c_byte) * 8)]
-short = globals()["int%d" % (ctypes.sizeof(ctypes.c_short) * 8)]
-ushort = globals()["uint%d" % (ctypes.sizeof(ctypes.c_ushort) * 8)]
-int_ = globals()["uint%d" % (ctypes.sizeof(ctypes.c_int) * 8)]
-uint = globals()["uint%d" % (ctypes.sizeof(ctypes.c_uint) * 8)]
-long_ = globals()["int%d" % (ctypes.sizeof(ctypes.c_long) * 8)]
-ulong = globals()["uint%d" % (ctypes.sizeof(ctypes.c_ulong) * 8)]
-longlong = globals()["int%d" % (ctypes.sizeof(ctypes.c_longlong) * 8)]
-ulonglong = globals()["uint%d" % (ctypes.sizeof(ctypes.c_ulonglong) * 8)]
+
+_make_signed = lambda x: globals()["int%d" % (numpy.dtype(x).itemsize * 8)]
+_make_unsigned = lambda x: globals()["uint%d" % (numpy.dtype(x).itemsize * 8)]
+
+char = _make_signed(numpy.byte)
+uchar = byte = _make_unsigned(numpy.byte)
+short = _make_signed(numpy.short)
+ushort = _make_unsigned(numpy.short)
+int_ = _make_signed(numpy.int_)
+uint = _make_unsigned(numpy.int_)
+long_ = _make_signed(numpy.long)
+ulong = _make_unsigned(numpy.long)
+longlong = _make_signed(numpy.longlong)
+ulonglong = _make_unsigned(numpy.longlong)
 
 
 __all__ = '''
